#ifndef _DELAY_ASM_
#define _DELAY_ASM_

;=======================================================================;
; Busy-wait functions                                                   ;
;                                                                       ;
; Target mcu:   Atmel AVR ATmega8                                       ;
; Mcu clock:    12 Mhz                                                  ;
; Author:       Viliam Sabol (sabolv@gmail.com)                         ;
;                                                                       ;
;=======================================================================;

.include "m8def.inc"
	
	; Delays for specified number(in r16 register) of ms + 22MCU cycles.
	; 	- delay time including call and ret instructions.
	;	- for MCU with fosc = 12Mhz
	DELAYMS:
		push r16				;{8 cycles}
		push r17
		push r18
		push r19
	  	_DELAYMS_START:
			ldi r17, 0xCF		;{1 cycle}	load low byte of iteration variable (0x7CF = 1999) to reg.
			ldi r18, 0x07		;{1 cycle}	load high byte ...
			_DELAYMS_MSCYCLE:	;{6*1999 - 1 cycles = 11993}
				subi r17, 0x01	;{1 cycle}	subtract 1 from iteration variable - low byte	
				sbci r18, 0x00	;{1 cycle}	subtract with carry high byte
				mov r19, r18	;{1 cycle}	copy high byte to r18 reg
				or r19, r17		;{1 cycle}	make or and jump if the result is not zero
				brne _DELAYMS_MSCYCLE	;{1 cycle or 2 cycles if condition is false}
			dec r16				;{1 cycle}
			nop					;{2 cycles} to fill 12000 cycles
			nop
		    brne _DELAYMS_START	;{1 cycle or 2 cycles if condition is false}
		pop r19					;{8 cycles}
		pop r18
		pop r17
		pop r16
		ret	 					;{4 cycles}

	; Delays for specified number(in r16 register) of us + 10MCU cycles.
	; 	- delay time including call and ret instructions.
	;	- for MCU with fosc = 12Mhz
	DELAYUS:					;{3 cycles for rcall}
		push r16				;{2 cycles}
		_DELAYUS_START:
		    nop					;{9 cycles}
			nop
			nop
			nop
			nop
			nop
			nop
			nop
			nop
		  	dec r16				;{1 cycle}
		  	brne _DELAYUS_START	;{1 cycle or 2 cycles if condition is false}
		pop r16					;{2 cycles}
	  	ret						;{4 cycles}

#endif /* _DELAY_ASM_ */
